/* Jungle Monkey
 * Copyright (C) 1999-2001  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "jmann.h"
#include "jmchannel_debug.h"


JMAnn*  
jmann_new (const gchar* name)
{
  JMAnn* ann;

  g_return_val_if_fail (name, NULL);

  ann = g_new0 (JMAnn, 1);
  ann->name = g_strdup (name);

  return ann;
}


void
jmann_delete (JMAnn* ann)
{
  if (!ann)
    return;

  g_free (ann->name);
  g_free (ann->path);
  gnet_url_delete (ann->url);

  elf_delete (ann->elf);

  g_free (ann);
}


JMAnn*	 
jmann_get_child (JMAnn* ann, const gchar* name)
{
  GList* i;

  g_return_val_if_fail (ann,  NULL);
  g_return_val_if_fail (name, NULL);

  for (i = ann->kids; i != NULL; i = i->next)
    {
      JMAnn* kid = (JMAnn*) i->data;

      if (!strcmp (kid->name, name))
	return kid;
    }

  return NULL;
}


gchar*
jmann_make_path (JMAnn* parent, const gchar* name)
{
  gchar* name_esc;
  gchar* path;

  g_return_val_if_fail (parent, NULL);
  g_return_val_if_fail (name, NULL);

  name_esc = stresc (name, "/\\", '\\');
  if (parent->path[0] == '/' && parent->path[1] == '\0') /* parent == root? */
    path = g_strconcat ("/", name_esc, NULL);
  else
    path = g_strconcat (parent->path, "/", name_esc, NULL);
  g_free (name_esc);

  return path;
}



/* ******************** */

void
jmann_set_attr_url (JMAnn* ann, const gchar* name, const GURL* value)
{
  gchar* str;

  g_return_if_fail (ann);
  g_return_if_fail (name);
  g_return_if_fail (value);

  str = gnet_url_get_nice_string (value);
  jmann_set_attr (ann, name, str);
  g_free (str);
}

void
jmann_set_attr_sha (JMAnn* ann, const gchar* name, const GSHA* value)
{
  gchar str[2 * GNET_SHA_HASH_LENGTH + 1];

  g_return_if_fail (ann);
  g_return_if_fail (name);
  g_return_if_fail (value);

  gnet_sha_copy_string (value, str);
  str[2 * GNET_SHA_HASH_LENGTH] = '\0';
  jmann_set_attr (ann, name, str);
}


GURL*	 
jmann_get_attr_url (const JMAnn* ann, const gchar* name)
{
  gchar* str;
  GURL*  url;

  g_return_val_if_fail (ann,  NULL);
  g_return_val_if_fail (name, NULL);

  str = jmann_get_attr (ann, name);
  if (!str) return NULL;

  url = gnet_url_new (str);
  return url;
}


GSHA*	 
jmann_get_attr_sha (const JMAnn* ann, const gchar* name)
{
  gchar* str;
  GSHA*  sha;

  g_return_val_if_fail (ann,  NULL);
  g_return_val_if_fail (name, NULL);

  str = jmann_get_attr (ann, name);
  if (!str) return NULL;
  if (strlen(str) != (GNET_SHA_HASH_LENGTH * 2))
    g_warning ("Bad SHA string %s\n", str);

  sha = gnet_sha_new_string (str);
  return sha;
}
