/* Jungle Monkey
 * Copyright (C) 1999-2001  The Regents of the University of Michigan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _JMCHANNEL_H
#define _JMCHANNEL_H

#include <glib.h>
#include <gnet/gnet.h>

#include "util/util.h"
#include "btp/btp.h"
#include "mtp/mtp_server.h"

#include "jmann.h"
#include "jmindex.h"


typedef struct _JMChannel JMChannel;

#define JMCHANNEL_UPDATE_TIME	 		       180000
#define JMCHANNEL_UPDATE_TIME_MAX    		        10000
#define JMCHANNEL_UPDATE_DELAY	  		    	 1000
#define JMCHANNEL_TIMEOUT_TIME	  (3 * JMCHANNEL_UPDATE_TIME)
#define JMCHANNEL_MAX_PACKET		  		 1400

#define JMCHANNEL_MIN_INDEX_SIZE	     		   16
#define JMCHANNEL_MAX_INDEX_SIZE			  128


typedef enum 
{
  JMCHANNEL_UPDATE,
  JMCHANNEL_REMOVE,
  JMCHANNEL_REMOVE_TIMEOUT,

} JMChannelType;


typedef void (*JMChannelInfoFunc)(JMChannel* channel, JMChannelType type, 
				  JMAnn* ann, gpointer user_data);
typedef void (*JMChannelErrorFunc)(JMChannel* channel, gpointer user_data);



struct _JMChannel
{
  BPeer*      peer;
  Btp* 	      btp;		/* BTP				  */
  GURL*	      url;		/* URL of channel		  */
  gboolean    is_local;		/* Is channel local (we created)  */
  gboolean    is_down;		/* Is channel down		  */

  gboolean    is_channels_only;	/* Allow channels in channel only */

  guint       update_timer;	/* Timer for sending updates	  */
  Timeval     update_tv;	/* Time last update was sent	  */
  guint       gc_timer;		/* Timer for garbage collection	  */

  JMAnn*      root_ann;		/* Root announcement (for channel)*/
  GHashTable* path_to_ann;	/* Name to announcement hash	  */

  GList*      local_anns;	/* Local channel announcements	  */
  guint       num_local_anns;	/* Number of local announcements  */
  GList*      send_anns;	/* Announcement to send		  */

  JMIndex*    cur_index;	/* Index file			  */
  GList*      indexes;		/* Indexes			  */
  GHashTable* ann_to_index;	/* Index ann to index		  */
  GURL*	      mtp_url;		/* URL of mirror rendezvous (opt) */
  MtpServer*  mtp_server;	/* Server for mirroring (opt)     */

  GHashTable* ann_to_rindex;	/* Remote indices		  */

  JMChannelInfoFunc  info_func; /* Channel info callback	  */
  JMChannelErrorFunc error_func;/* Channel error callback	  */
  gpointer 	     user_data; /* User data			  */

};


JMChannel* jmchannel_create (BPeer* bpeer, const gchar* name);
JMChannel* jmchannel_join (BPeer* bpeer, const GURL* url);
JMChannel* jmchannel_setup (BPeer* bpeer, const GURL* url);
gboolean   jmchannel_rejoin (JMChannel* channel); /* FALSE = success */
void 	   jmchannel_leave (JMChannel* channel);
void       jmchannel_delete (JMChannel* channel);

#define	   jmchannel_get_name(C) ((C)->url->resource)

#define    jmchannel_is_up(C)    ((C)->btp != NULL)
#define    jmchannel_is_down(C)  ((C)->is_down)

void	   jmchannel_set_mtp (JMChannel* channel, 
			      GURL* mtp_url, MtpServer* mtp_server);

JMAnn*	   jmchannel_get (JMChannel* channel, const gchar* path);
#define	   jmchannel_has(C, F)   (jmchannel_get(C,F) != NULL)

#define	   jmchannel_add(C,P,N,U) jmchannel_add_elf(C,P,N,U,NULL)
#define	   jmchannel_add_path(C,F,U) jmchannel_add_path_elf(C,F,U,NULL)

JMAnn*	   jmchannel_add_elf (JMChannel* channel, JMAnn* parent, 
			      const gchar* name, const GURL* url,
			      const ElfNode* elf);
JMAnn*	   jmchannel_add_path_elf (JMChannel* channel, const gchar* path, 
				   const GURL* url, const ElfNode* elf);

void	   jmchannel_remove (JMChannel* channel, JMAnn* ann);

void	   jmchannel_recv (JMChannel* channel, const void* buf, guint len);

#endif /* _JMCHANNEL_H */
